pro read_lunar, filename, status, lunar

;  Purpose:
;  This program provides the data structures necessary to read the
;  SAGE III Lunar Event Species Data binary product files.
;
;  Author:
;  Karen H. Sage
;  
;  Contact:
;  michael.l.heitz@nasa.gov 
;  
;  Revision History:
;  11/01/2000 - Initial Coding.
;  08/01/01 - Deleted variable definitions and read of pressure
;             surface variables because of change in file format.
;  10/16/02 - Changed lunar structure to an anonymous structure.
;  03/12/03 - Renamed Director Version and Solar Version to 
;             Software Version and Dataproduct Version.
;  09/19/03 - Added check of versions and new variables (EvtStatusFlag and
;             AbandAltRegOffset) to lunar structure.  Made code backward 
;             compatible.
;
;  Input variables:  the name of the lunar product file (the product 
;     file names have the form  g3a.lspb.*).
;  Output variables:  a read file status variable and the structure
;     containing the lunar product data.  See the SAGE III Data Products 
;     Users Guide for a description of the lunar product data.

;  Initialize local variables.

status = 'OK'
IEEE_flag = 0

itest = 512
ltest = 102580L
ftest = 1.23e10
dtest = 1.23d50

;  Determine if the current machine uses IEEE, big_endian numbers.
;    A sample int, long, float and double are converted using
;    byteorder and compared with the original.  If there is no 
;    change, the machine is assumed to be IEEE compliant and big-endian.
;    

it2 = itest
lt2 = ltest
ft2 = ftest
dt2 = dtest

byteorder, it2, /htons
byteorder, lt2, /htonl
byteorder, ft2, /ftoxdr
byteorder, dt2, /dtoxdr

case 1 of
(itest eq it2) and (ltest eq lt2) and (ftest eq ft2) and (dtest eq dt2) : IEEE_flag = 1
else:  IEEE_flag = 0
endcase

;  Create the structure for read from binary file of the data needed
;  to dynamically size data arrays.

lunar_size = { lunar_size, event_id:0L, $
          yr_day_tag:0L, $
          mission_time:0., $
          int_fill_value:0L, $
          flt_fill_value:0., $
          mission_id:0L, $
          L0DO_Version:0., $
          L0_Version:0., $
          Software_Version:0., $
          Dataproduct_Version:0., $
          DB_Version:0., $
          GRAM95_Version:0., $
          Met_Version:0., $
          Lun_Model_Ver:0., $
          Lun_Albedo_Ver:0., $
          bin_height:0., $
          num_alt_bins:0L, $
          num_press_grid:0L, $
          num_grnd_trk:0L }


;  Open file for direct read-only access.

on_ioerror, open_error
openr, in_unit, filename, /get_lun

;  Read size structure.

readu, in_unit, lunar_size

;  If machine is not IEEE then swap bytes to make them compliant.
 
if IEEE_flag eq 0 then lunar_size = swap_endian (lunar_size)
vers = lunar_size.Software_Version

;  Create the structure for read of all data from binary file.
if vers lt 3.00 then $
  lunar_temp = { Event_ID:0L, $
     yr_day_tag:0L, $
     mission_time:0., $
     int_fill_value:0L, $
     flt_fill_value:0., $
     mission_id:0L, $
     L0DO_Version:0., $
     L0_Version:0., $
     Software_Version:0., $
     Dataproduct_Version:0., $
     DB_Version:0., $
     GRAM95_Version:0., $
     Met_Version:0., $
     Lun_Model_Ver:0., $
     Lun_Albedo_Ver:0., $
     bin_height:0., $
     num_alt_bins:0L, $
     num_press_grid:0L, $
     num_grnd_trk:0L, $
     sc_evt_type:0L, $
     gnd_evt_type:0L, $
     BetaAngle:0., $
     LunarPhase:0., $
     ZenithAngle:0., $
     aurora_flag:0L, $
     st_date:0L, $
     st_time:0L, $
     st_latitude:0., $
     st_longitude:0., $
     st_altitude:0., $
     end_date:0L, $
     end_time:0L, $
     end_latitude:0., $
     end_longitude:0., $
     end_altitude:0., $
     gt_date:lonarr(lunar_size.num_grnd_trk), $
     gt_time:lonarr(lunar_size.num_grnd_trk), $
     gt_latitude:fltarr(lunar_size.num_grnd_trk), $
     gt_longitude:fltarr(lunar_size.num_grnd_trk), $
     gt_ray_dir:fltarr(lunar_size.num_grnd_trk), $
     trop_temp:0., $
     trop_alt:0., $
     met_pressure:fltarr(lunar_size.num_press_grid), $
     met_temp:fltarr(lunar_size.num_press_grid), $
     met_temp_unc:fltarr(lunar_size.num_press_grid), $
     met_altitude:fltarr(lunar_size.num_press_grid), $
     Met_Source:0L, $
     geom_alt:fltarr(lunar_size.num_alt_bins), $
     AltRegQA:lonarr(lunar_size.num_alt_bins), $
     potential_alt:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp_uncert:fltarr(lunar_size.num_alt_bins), $
     CurrentPress:fltarr(lunar_size.num_alt_bins), $
     CurrentPress_uncert:fltarr(lunar_size.num_alt_bins), $
     TPSource:lonarr(lunar_size.num_alt_bins), $
     CompositeO3:fltarr(lunar_size.num_alt_bins), $
     CompositeO3_uncert:fltarr(lunar_size.num_alt_bins), $
     CompO3QA:lonarr(lunar_size.num_alt_bins), $
     NO2NDP:fltarr(lunar_size.num_alt_bins), $
     NO2NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO2QA:lonarr(lunar_size.num_alt_bins), $
     NO3NDP:fltarr(lunar_size.num_alt_bins), $
     NO3NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO3QA:lonarr(lunar_size.num_alt_bins), $
     OClONDP:fltarr(lunar_size.num_alt_bins), $
     OClONDP_uncert:fltarr(lunar_size.num_alt_bins), $
     OClOQA:lonarr(lunar_size.num_alt_bins) }

                          ; First product version.

lunar = { Event_ID:0L, $
     yr_day_tag:0L, $
     mission_time:0., $
     int_fill_value:0L, $
     flt_fill_value:0., $
     mission_id:0L, $
     L0DO_Version:0., $
     L0_Version:0., $
     Software_Version:0., $
     Dataproduct_Version:0., $
     DB_Version:0., $
     GRAM95_Version:0., $
     Met_Version:0., $
     Lun_Model_Ver:0., $
     Lun_Albedo_Ver:0., $
     bin_height:0., $
     num_alt_bins:0L, $
     num_press_grid:0L, $
     num_grnd_trk:0L, $
     sc_evt_type:0L, $
     gnd_evt_type:0L, $
     BetaAngle:0., $
     LunarPhase:0., $
     ZenithAngle:0., $
     aurora_flag:0L, $
     EvtStatusFlag:0L, $
     st_date:0L, $
     st_time:0L, $
     st_latitude:0., $
     st_longitude:0., $
     st_altitude:0., $
     end_date:0L, $
     end_time:0L, $
     end_latitude:0., $
     end_longitude:0., $
     end_altitude:0., $
     gt_date:lonarr(lunar_size.num_grnd_trk), $
     gt_time:lonarr(lunar_size.num_grnd_trk), $
     gt_latitude:fltarr(lunar_size.num_grnd_trk), $
     gt_longitude:fltarr(lunar_size.num_grnd_trk), $
     gt_ray_dir:fltarr(lunar_size.num_grnd_trk), $
     trop_temp:0., $
     trop_alt:0., $
     met_pressure:fltarr(lunar_size.num_press_grid), $
     met_temp:fltarr(lunar_size.num_press_grid), $
     met_temp_unc:fltarr(lunar_size.num_press_grid), $
     met_altitude:fltarr(lunar_size.num_press_grid), $
     Met_Source:0L, $
     geom_alt:fltarr(lunar_size.num_alt_bins), $
     AltRegQA:lonarr(lunar_size.num_alt_bins), $
     potential_alt:fltarr(lunar_size.num_alt_bins), $
     AbandAltRegOffset:0., $
     CurrentTemp:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp_uncert:fltarr(lunar_size.num_alt_bins), $
     CurrentPress:fltarr(lunar_size.num_alt_bins), $
     CurrentPress_uncert:fltarr(lunar_size.num_alt_bins), $
     TPSource:lonarr(lunar_size.num_alt_bins), $
     CompositeO3:fltarr(lunar_size.num_alt_bins), $
     CompositeO3_uncert:fltarr(lunar_size.num_alt_bins), $
     CompO3QA:lonarr(lunar_size.num_alt_bins), $
     NO2NDP:fltarr(lunar_size.num_alt_bins), $
     NO2NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO2QA:lonarr(lunar_size.num_alt_bins), $
     NO3NDP:fltarr(lunar_size.num_alt_bins), $
     NO3NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO3QA:lonarr(lunar_size.num_alt_bins), $
     OClONDP:fltarr(lunar_size.num_alt_bins), $
     OClONDP_uncert:fltarr(lunar_size.num_alt_bins), $
     OClOQA:lonarr(lunar_size.num_alt_bins) }

;  Change file pointer to beginning of file and read in complete file
;  structure.

point_lun, in_unit, 0

if vers lt 3.00 then readu, in_unit, lunar_temp $
else  readu, in_unit, lunar

if vers lt 3.00 then BEGIN
     lunar.Event_ID = lunar_temp.Event_ID
     lunar.yr_day_tag = lunar_temp.yr_day_tag 
     lunar.mission_time = lunar_temp.mission_time
     lunar.int_fill_value = lunar_temp.int_fill_value
     lunar.flt_fill_value = lunar_temp.flt_fill_value
     lunar.mission_id = lunar_temp.mission_id
     lunar.L0DO_Version = lunar_temp.L0DO_Version 
     lunar.L0_Version = lunar_temp.L0_Version
     lunar.Software_Version = lunar_temp.Software_Version
     lunar.Dataproduct_Version = lunar_temp.Dataproduct_Version
     lunar.DB_Version = lunar_temp.DB_Version 
     lunar.GRAM95_Version = lunar_temp.GRAM95_Version
     lunar.Met_Version = lunar_temp.Met_Version 
     lunar.Lun_Model_Ver = lunar_temp.Lun_Model_Ver
     lunar.Lun_Albedo_Ver = lunar_temp.Lun_Albedo_Ver
     lunar.bin_height = lunar_temp.bin_height
     lunar.num_alt_bins = lunar_temp.num_alt_bins
     lunar.num_press_grid = lunar_temp.num_press_grid 
     lunar.num_grnd_trk = lunar_temp.num_grnd_trk
     lunar.sc_evt_type = lunar_temp.sc_evt_type
     lunar.gnd_evt_type = lunar_temp.gnd_evt_type
     lunar.BetaAngle = lunar_temp.BetaAngle
     lunar.LunarPhase = lunar_temp.LunarPhase
     lunar.ZenithAngle = lunar_temp.ZenithAngle
     lunar.aurora_flag = lunar_temp.aurora_flag
     lunar.EvtStatusFlag = lunar_temp.int_fill_value
     lunar.st_date = lunar_temp.st_date
     lunar.st_time = lunar_temp.st_time
     lunar.st_latitude = lunar_temp.st_latitude
     lunar.st_longitude = lunar_temp.st_longitude
     lunar.st_altitude = lunar_temp.st_altitude
     lunar.end_time = lunar_temp.end_time
     lunar.end_latitude = lunar_temp.end_latitude
     lunar.end_longitude = lunar_temp.end_longitude
     lunar.end_altitude = lunar_temp.end_altitude
     lunar.gt_date = lunar_temp.gt_date
     lunar.gt_time = lunar_temp.gt_time
     lunar.gt_latitude = lunar_temp.gt_latitude
     lunar.gt_longitude = lunar_temp.gt_longitude
     lunar.gt_ray_dir = lunar_temp.gt_ray_dir
     lunar.trop_temp = lunar_temp.trop_temp
     lunar.trop_alt = lunar_temp.trop_alt 
     lunar.met_pressure = lunar_temp.met_pressure
     lunar.met_temp = lunar_temp.met_temp
     lunar.met_temp_unc = lunar_temp.met_temp_unc
     lunar.met_altitude = lunar_temp.met_altitude
     lunar.Met_Source = lunar_temp.Met_Source
     lunar.geom_alt = lunar_temp.geom_alt
     lunar.AltRegQA = lunar_temp.AltRegQA
     lunar.potential_alt = lunar_temp.potential_alt
     lunar.AbandAltRegOffset = lunar_temp.flt_fill_value
     lunar.CurrentTemp = lunar_temp.CurrentTemp
     lunar.CurrentTemp_uncert = lunar_temp.CurrentTemp_uncert
     lunar.CurrentPress = lunar_temp.CurrentPress
     lunar.CurrentPress_uncert = lunar_temp.CurrentPress_uncert
     lunar.TPSource = lunar_temp.TPSource
     lunar.CompositeO3 = lunar_temp.CompositeO3
     lunar.CompositeO3_uncert = lunar_temp.CompositeO3_uncert
     lunar.CompO3QA = lunar_temp.CompO3QA
     lunar.NO2NDP = lunar_temp.NO2NDP
     lunar.NO2NDP_uncert = lunar_temp.NO2NDP_uncert
     lunar.NO2QA = lunar_temp.NO2QA
     lunar.NO3NDP = lunar_temp.NO3NDP
     lunar.NO3NDP_uncert = lunar_temp.NO3NDP_uncert
     lunar.NO3QA = lunar_temp.NO3QA
     lunar.OClONDP = lunar_temp.OClONDP
     lunar.OClONDP_uncert = lunar_temp.OClONDP_uncert
     lunar.OClOQA = lunar_temp.OClOQA
ENDIF

;  If machine is not IEEE then swap bytes to make them compliant.
if IEEE_flag eq 0 then lunar = swap_endian (lunar)

;  Close data file unit and free the logical unit number.
;openw,1,'testalt0711',/append
;printf,1,lunar.Event_ID, lunar.sc_evt_type, lunar.LunarPhase, lunar.BetaAngle, lunar.ZenithAngle, lunar.Ab;andAltRegOffset
;close,1

;stop,323
close, in_unit
free_lun, in_unit
goto, program_end

open_error:
print, 'Error on open or read of file ', filename
status = 'ERROR'

program_end:

end
