"""
Author:      David Huber, SSAI <david.b.huber@nasa.gov>
Date:        2/11/2016
Description: This module houses functions that read in SAGE III lunar L2 product
             files and return a data object containing the metadata and products
             for the given file type.  The functions are described below.
"""

import numpy as np
from sys import version_info

#Declare which symbols to import
__version = version_info
if(__version[0] == 2):
   __all__ = ["LunL2Data", "lun_l2_hdf4", "lun_l2_bin", "lun_l2_hdf5"]
elif(__version[0] == 3):
   #Do not import the HDF4 readers for Python version 3.x.  A required module,
   #pyhdf, is not available for version 3 (at least at the time that this
   #module was written).
   __all__ = ["LunL2Data", "lun_l2_bin", "lun_l2_hdf5"]
else:
   print("Import error, unable to determine Python version")

#Set global constants
__c_MaxIndex = 20

class LunL2Data:
   """
   Class Name: LunL2Data
   Purpose:    The class contains the information from a given level 2 SAGE III
               data file.  This structure is the same for binary, HDF4, and HDF5
               files.
   """
   def __init__(self, num_grnd_trk, num_alt_bins, num_press_grid):

      #Event information
      self.Event_ID = 0
      self.yr_day_tag = 0
      self.mission_time = 0
      #Fill values
      self.int_fill_value = 0
      self.flt_fill_value = 0.
      #Mission ID
      self.mission_id = 0
      #Software and data versions
      self.L0DO_Version = 0.
      self.L0_Version = 0.
      self.DB_Version = 0.
      self.GRAM95_Version = 0.
      self.Met_Version = 0.
      self.Software_Version = 0.
      self.Dataproduct_Version = 0.
      self.Lun_Model_Ver = 0.
      self.Lun_Albedo_Ver = 0.
      #Bin height
      self.bin_height = 0.
      #Dimensions
      self.num_alt_bins = num_alt_bins
      self.num_press_grid = num_press_grid
      self.num_grnd_trk = num_grnd_trk
      #Event type
      self.sc_evt_type = 0
      self.gnd_evt_type = 0
      #Lunar beta angle
      self.BetaAngle = 0.
      #Observational information
      self.LunarPhase = 0.
      self.ZenithAngle = 0.
      self.aurora_flag = 0
      #Status flag
      self.EvtStatusFlag = 0
      #Starting time/location
      self.st_date = 0
      self.st_time = 0
      self.st_latitude = 0.
      self.st_longitude = 0.
      self.st_altitude = 0.
      #Ending time/location
      self.end_date = 0
      self.end_time = 0
      self.end_latitude = 0.
      self.end_longitude = 0.
      self.end_altitude = 0.
      #Ground track information
      self.gt_date = np.zeros((num_grnd_trk), dtype = np.int)
      self.gt_time = np.zeros((num_grnd_trk), dtype = np.int)
      self.gt_latitude = np.zeros((num_grnd_trk), dtype = np.float)
      self.gt_longitude = np.zeros((num_grnd_trk), dtype = np.float)
      self.gt_ray_dir = np.zeros((num_grnd_trk), dtype = np.float)
      #Tropopause temperature/altitude
      self.trop_temp = 0.
      self.trop_alt = 0.
      self.Met_Source = 0
      #Meteorological inputs
      self.met_pressure = np.zeros((num_press_grid), dtype = np.float)
      self.met_temp = np.zeros((num_press_grid), dtype = np.float)
      self.met_temp_unc = np.zeros((num_press_grid), dtype = np.float)
      self.met_altitude = np.zeros((num_press_grid), dtype = np.float)
      #Altitude registration
      self.geom_alt = np.zeros((num_alt_bins), dtype = np.float)
      self.AltRegQA = np.zeros((num_alt_bins), dtype = np.int)
      self.potential_alt = np.zeros((num_alt_bins), dtype = np.float)
      #A-Band altitude offset
      self.AbandAltOffset = 0.
      #Retrieved T/P
      self.CurrentTemp = np.zeros((num_alt_bins), dtype = np.float)
      self.CurrentTemp_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.CurrentPress = np.zeros((num_alt_bins), dtype = np.float)
      self.CurrentPress_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.TPSource = np.zeros((num_alt_bins), dtype = np.int)
      #Composite derived ozone products
      self.CompositeO3 = np.zeros((num_alt_bins), dtype = np.float)
      self.CompositeO3_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.CompO3QA = np.zeros((num_alt_bins), dtype = np.int)
      #Nitrogen dioxide products
      self.NO2NDP = np.zeros((num_alt_bins), dtype = np.float)
      self.NO2NDP_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.NO2QA = np.zeros((num_alt_bins), dtype = np.int)
      #Nitrogen trioxide products
      self.NO3NDP = np.zeros((num_alt_bins), dtype = np.float)
      self.NO3NDP_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.NO3QA = np.zeros((num_alt_bins), dtype = np.int)
      #Chlorine dioxide products
      self.OClONDP = np.zeros((num_alt_bins), dtype = np.float)
      self.OClONDP_uncert = np.zeros((num_alt_bins), dtype = np.float)
      self.OClOQA = np.zeros((num_alt_bins), dtype = np.int)


##############################################################


def __l_read_int32(File, Count):
   """
   Name:        __l_read_int32
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      1) File handle or filename, 2) number of bytes to read in
                3) number of times to read
   Output:      The integer or integer array read in from the file
   Description: Reads in a 4-byte integer from a specified binary file on a
                little-endian system.
   """

   return np.fromfile(File, dtype=np.int32, count = Count).newbyteorder()


##############################################################


def __l_read_float32(File, Count):
   """
   Name:        __l_read_float32
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      1) File handle or filename, 2) size of 1-d array
   Output:      The integer or integer array read in from the file
   Description: Reads in a 4-byte floating point array from a specified binary
                file on a little-endian system.
   """

   return np.fromfile(File, dtype=np.float32, count = Count).newbyteorder()


##############################################################


def __b_read_int32(File, Count):
   """
   Name:        __b_read_int32
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      1) File handle or filename, 2) number of bytes to read in
                3) number of times to read
   Output:      The integer or integer array read in from the file
   Description: Reads in a 4-byte integer from a specified binary file on a
                big-endian system.
   """

   return np.fromfile(File, dtype=np.int32, count = Count).newbyteorder()


##############################################################


def __b_read_float32(File, Count):
   """
   Name:        __b_read_float32
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      1) File handle or filename, 2) size of 1-d array
   Output:      The integer or integer array read in from the file
   Description: Reads in a 4-byte floating point array from a specified binary
                file on a big-endian system.
   """

   return np.fromfile(File, dtype=np.float32, count = Count).newbyteorder()


##############################################################


def lun_l2_hdf5(Filename):
   """
   Name:        lun_l2_hdf5
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      Filename of a lunar level 2 HDF5 product file
   Outputs:     1) A data object of type LunTranData containing the product data
                2) A status flag indicating whether the read was successful or not
   Description: This function reads the data from a lunar HDF5 level 2 product
                file and returns a data object containing the data from the file.
                A status flag is also returned and should be checked before
                attempting to access the structure after function completion.  A
                status value of 0 indicates a successful read while a value of -1
                indicates that an error occurred.
   """

   #Import h5py for interaction with HDF5 files
   import h5py

   #Determine if the input file exists
   try:
      File = h5py.File(Filename, "r")
   except OSError:
      print(Filename, "is not an HDF5 file. Exiting...\n")
      return(LunL2Data(1,1,1), -1)

   #Go ahead and declare the section names to avoid confusion later on when
   #reading in the file contents.
   Section1 = "Section 1.0 - File Header/Section 1.0 - File Header"
   Section2 = "Section 2.0 - Product Identification/Section 2.0 - Product Identification"
   Section3 = "Section 3.0 - File Description/File Description"
   Section4Root = "Section 4.0 - Event Identification"
   Section4 = Section4Root + "/Event Identification"
   Section41 = (Section4Root + "/Section 4.1 - Science Data Start Information" +
                "/Section 4.1 - Science Data Start Information")
   Section42 = (Section4Root + "/Section 4.2 - Science Data End Information" +
                "/Section 4.2 - Science Data End Information")
   Section43 = (Section4Root + "/Section 4.3 - Ground Track Data Over This Event" +
                "/Section 4.3 - Ground Track Data Over This Event")
   Section5 = "Section 5.0 - Tropopause Data/Tropopause Data"
   Section51 = "Section 5.1 - Meteorological Data/Meteorological Data"
   Section6 = "Section 6.0 - Altitude-based array data/Altitude-based data"
   Section6A = "Section 6.0A - Altitude-based scalar data/A-band Altitude Registration Offset"
   Section61 = "Section 6.1 - Temperature pressure profiles/Temperature pressure profiles"
   Section62 = "Section 6.2 - Ozone profiles/Ozone profiles"
   Section63 = "Section 6.3 - Nitrogen Dioxide profiles/Nitrogen Dioxide profiles"
   Section64 = "Section 6.4 - Nitrogen Trioxide profiles/Nitrogen Trioxide profiles"
   Section65 = "Section 6.5 - OClO profiles/OClO profiles"

   #Make sure that we have the correct file type by checking whether all datasets exist
   e1 = Section1 in File
   e2 = Section2 in File
   e3 = Section3 in File
   e4 = Section4 in File
   e41 = Section41 in File
   e42 = Section42 in File
   e43 = Section43 in File
   e5 = Section5 in File
   e51 = Section51 in File
   e6 = Section6 in File
   e6a = Section6A in File
   e61 = Section61 in File
   e62 = Section62 in File
   e63 = Section63 in File
   e64 = Section64 in File
   e65 = Section65 in File

   if(not(e1 & e2 & e3 & e4 & e41 & e42 & e43 & e5 & e51 & e6a & e61 & e62 & e63 &
        e64 & e65)):
      print("Error reading " + str(Filename) + ": Unable to locate all SAGE datasets. Exiting...\n")
      errlun = LunL2Data(1, 1, 1)
      return(errlun, -1)

   #Get the file description handle
   DataFile = File[Section3][0]
   num_alt_bins = DataFile[1]
   num_grnd_trk = DataFile[2]
   num_press_grid = DataFile[3]

   #Declare the output data object
   lunar = LunL2Data(num_grnd_trk, num_alt_bins, num_press_grid)

   #Get the height of each bin
   lunar.bin_height = DataFile[0]

   #Get the header information handle
   DataHeader = File[Section1][0]
   lunar.Event_ID = DataHeader[0]
   lunar.yr_day_tag = DataHeader[1]
   lunar.mission_time = DataHeader[2]
   lunar.int_fill_value = DataHeader[3]
   lunar.flt_fill_value = DataHeader[4]
   lunar.mission_id = DataHeader[5]

   #Get the product information handle
   DataProduct = File[Section2][0]
   lunar.L0DO_Version = DataProduct[0]
   lunar.L0_Version = DataProduct[1]
   lunar.DB_Version = DataProduct[2]
   lunar.GRAM95_Version = DataProduct[3]
   lunar.Met_Version = DataProduct[4]
   lunar.Software_Version = DataProduct[5]
   lunar.Dataproduct_Version = DataProduct[6]
   lunar.Lun_Model_Ver = DataProduct[7]
   lunar.Lun_Albedo_Ver = DataProduct[8]

   #Get the event identification handle
   DataEvent = File[Section4][0]
   lunar.sc_evt_type = DataEvent[0]
   lunar.gnd_evt_type = DataEvent[1]
   lunar.BetaAngle = DataEvent[2]
   lunar.LunarPhase = DataEvent[3]
   lunar.ZenithAngle = DataEvent[4]
   lunar.aurora_flag = DataEvent[5]
   lunar.EvtStatusFlag = DataEvent[6]

   #Get the event start information handle
   DataEventStart = File[Section41][0]
   lunar.st_date = DataEventStart[0]
   lunar.st_time = DataEventStart[1]
   lunar.st_latitude = DataEventStart[2]
   lunar.st_longitude = DataEventStart[3]
   lunar.st_altitude = DataEventStart[4]

   #Get the event end infromation handle
   DataEventEnd = File[Section42][0]
   lunar.end_date = DataEventEnd[0]
   lunar.end_time = DataEventEnd[1]
   lunar.end_latitude = DataEventEnd[2]
   lunar.end_longitude = DataEventEnd[3]
   lunar.end_altitude = DataEventEnd[4]

   #Get the event ground track information handle
   DataEventGround = np.asarray(File[Section43])
   lunar.gt_date = DataEventGround["Date (YYYYMMDD)"]
   lunar.gt_time = DataEventGround["Time (HHMMSS)"]
   lunar.gt_latitude = DataEventGround["Subtangent Latitude (0.0 +/- 90.0 deg)"]
   lunar.gt_longitude = DataEventGround["Subtangent Longitude (0.0 +/- 180.0 deg)"]
   lunar.gt_ray_dir = DataEventGround["Ray Path Direction @ Subtan. Pt. (0.0 - 359.99 deg)"]

   #Get the tropopause data handle
   DataTrop = File[Section5][0]
   lunar.trop_temp = DataTrop[0]
   lunar.trop_alt = DataTrop[1]
   lunar.Met_Source = DataTrop[2]

   #Get the meteorological input profiles handle
   DataMet = np.asarray(File[Section51])

   #Get the altitude registration product handle
   DataAlt = np.asarray(File[Section6])

   #Get the A-band altitude registration product handle
   DataABand = File[Section6A][0]
   lunar.AbandAltOffset = DataABand[0]

   #Get the temperature/pressure product handle
   DataTP = np.asarray(File[Section61])

   #Get the composite ozone product handle
   DataO3 = np.asarray(File[Section62])

   #Get the nitrogen dioxide product handle
   DataNO2 = np.asarray(File[Section63])

   #Get the nitrogen trioxide product handle
   DataNO3 = np.asarray(File[Section64])

   #Get the chlorine dioxide product handle
   DataOClO = np.asarray(File[Section65])

   #Populate the output data object
   lunar.met_pressure = DataMet['Met Pressure (hPa)']
   lunar.met_temp = DataMet['Met Temperature (K)']
   lunar.met_temp_unc = DataMet['Met Temperature Unc (K)']
   lunar.met_altitude = DataMet['Met Altitude (km)']

   lunar.geom_alt = DataAlt['Geometric Altitude (km)']
   lunar.AltRegQA = DataAlt['Altitude Registration QA Flags']
   lunar.potential_alt = DataAlt['Geopotential Altitude (km)']

   lunar.CurrentTemp = DataTP['Current Temperature (K)']
   lunar.CurrentTemp_uncert = DataTP['Current Temperature Unc (K)']
   lunar.CurrentPress = DataTP['Current Pressure (hPa)']
   lunar.CurrentPress_uncert = DataTP['Current Pressure Unc (hPa)']
   lunar.TPSource = DataTP['Pressure/Temp. Array Source']

   lunar.CompositeO3 = DataO3['Ozone Concentration (cm-3)']
   lunar.CompositeO3_uncert = DataO3['Ozone Concentration Unc (cm-3)']
   lunar.CompO3QA = DataO3['Ozone QA bit flags']

   lunar.NO2NDP = DataNO2['Nitrogen Dioxide Concentration (cm-3)']
   lunar.NO2NDP_uncert = DataNO2['Nitrogen Dioxide Concentration Unc (cm-3)']
   lunar.NO2QA = DataNO2['Nitrogen Dioxide QA bit flags']

   lunar.NO3NDP = DataNO3['Nitrogen Trioxide Concentration (cm-3)']
   lunar.NO3NDP_uncert = DataNO3['Nitrogen Trioxide Concentration Unc (cm-3)']
   lunar.NO3QA = DataNO3['Nitrogen Trioxide QA bit flags']

   lunar.OClONDP = DataOClO['OClO Concentration  (cm-3)']
   lunar.OClONDP_uncert = DataOClO['OClO Concentration Unc (cm-3)']
   lunar.OClOQA = DataOClO['OClO QA bit flags']

   #Close the HDF5 file
   File.close()

   return (lunar, 0)


########################################################################


#Read HDF4 level 2 product files
def lun_l2_hdf4(Filename):
   """
   Name:        lun_l2_hdf4
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      The filename of a lunar level 2 HDF4 product file
   Outputs:     1) A data object of type LunL2Data containing the product data
                2) A status flag indicating whether the read was successful or not
   Description: This function reads the data from an HDF4 level 2 product file
                and returns a data object containing the data from the file.
                A status flag is also returned and should be checked before
                attempting to access the structure after function completion.  A
                status value of 0 indicates a success while a value of -1
                indicates that an error occurred.
   """
   #Import pyhdf for interaction with HDF4 files.  Note that pyhdf is ONLY
   #available for Python 2.x.  If you wish to work with SAGE data files in
   #Python 3.x, use the binary or HDF5 readers.

   try:
      from pyhdf import HDF, VS
   except ImportError
      if(__version[0] != 2):
         raise ImportError('HDF4 files are only supported in Python 2.x.' +
                           '  Please use the HDF5 product files if you must use Python 3.x')
      else:
         raise ImportError('Unable to locate the pyhdf module.' +
                           '  Please ensure version 0.9 or better is installed.')

   #Open the file
   FileHandle = HDF.HDF(Filename)
   DataInterface = FileHandle.vstart()

   #Find the starting index
   for Index in range(__c_MaxIndex+1):
      try:
         TestVD = DataInterface.attach(Index)
         TestVD.detach()
         break
      except HDF.HDF4Error:
         continue

   if(Index == __c_MaxIndex):
      print("Failed to locate the starting VData index for ", Filename)
      return(-1,-1)

   #Get "Header" vdata information (Not actually the file header)
   VDataHeader = DataInterface.attach(Index)
   Index += 2
   try:
      HeaderData = VDataHeader.read()
   except HDF.HDF4Error:
      VDataHeader.detach()
      print("Failed to read header data from the HDF file")
      return(-1,-1)
   VDataHeader.detach()

   #Get product identification vdata information
   VDataProdId = DataInterface.attach(Index)
   Index += 2
   try:
      ProdIdData = VDataProdId.read()
   except HDF.HDF4Error:
      VDataProdId.detach()
      print("Failed to read product information from the HDF file")
      return(-1,-1)
   VDataProdId.detach()

   #Get file information
   VDataFileInfo = DataInterface.attach(Index)
   Index += 2
   try:
      ProdFileInfo = VDataFileInfo.read()
   except HDF.HDF4Error:
      VDataFileInfo.detach()
      print("Failed to read file information from the HDF file")
      return(-1,-1)
   VDataFileInfo.detach()

   #Get event information
   VDataEvent = DataInterface.attach(Index)
   Index += 2
   try:
      ProdEvent = VDataEvent.read()
   except HDF.HDF4Error:
      VDataEvent.detach()
      print("Failed to read event information from the HDF file")
      return(-1,-1)
   VDataEvent.detach()

   #Get information on the start of science data
   VDataSciStart = DataInterface.attach(Index)
   Index += 2
   try:
      ProdSciStart = VDataSciStart.read()
   except HDF.HDF4Error:
      VDataSciStart.detach()
      print("Failed to read science starting information from the HDF file")
      return(-1,-1)
   VDataSciStart.detach()

   #Get information on the end of the event
   VDataSciEnd = DataInterface.attach(Index)
   Index += 2
   try:
      ProdSciEnd = VDataSciEnd.read()
   except HDF.HDF4Error:
      VDataSciEnd.detach()
      print("Failed to read science ending information from the HDF file")
      return(-1,-1)
   VDataSciEnd.detach()


   #Collect array size information to initiate the output lunar object
   num_alt_bins = ProdFileInfo[0][1]
   num_grnd_trk = ProdFileInfo[0][2]
   num_press_grid = ProdFileInfo[0][3]

   #Instatiate the lunar object
   lunar = LunL2Data(num_grnd_trk, num_alt_bins, num_press_grid)

   #Begin populating the object attributes
   #Populate header data
   lunar.Event_ID = HeaderData[0][0]
   lunar.yr_day_tag = HeaderData[0][1]
   lunar.mission_time = HeaderData[0][2]
   lunar.int_fill_value = HeaderData[0][3]
   lunar.flt_fill_value = HeaderData[0][4]
   lunar.mission_id = HeaderData[0][5]
   #General product information
   lunar.L0DO_Version = ProdIdData[0][0]
   lunar.L0_Version = ProdIdData[0][1]
   lunar.DB_Version = ProdIdData[0][2]
   lunar.GRAM95_Version = ProdIdData[0][3]
   lunar.Met_Version = ProdIdData[0][4]
   lunar.Software_Version = ProdIdData[0][5]
   lunar.Dataproduct_Version = ProdIdData[0][6]
   lunar.Lun_Model_Ver = ProdIdData[0][7]
   lunar.Lun_Albedo_Ver = ProdIdData[0][8]
   #Array sizes
   lunar.bin_height = ProdFileInfo[0][0]
   #Event information
   lunar.sc_evt_type = ProdEvent[0][0]
   lunar.gnd_evt_type = ProdEvent[0][1]
   lunar.BetaAngle = ProdEvent[0][2]
   lunar.LunarPhase = ProdEvent[0][3]
   lunar.ZenithAngle = ProdEvent[0][4]
   lunar.aurora_flag = ProdEvent[0][5]
   lunar.EvtStatusFlag = ProdEvent[0][6]
   #Event starting information
   lunar.st_date = ProdSciStart[0][0]
   lunar.st_time = ProdSciStart[0][1]
   lunar.st_latitude = ProdSciStart[0][2]
   lunar.st_longitude = ProdSciStart[0][3]
   lunar.st_altitude = ProdSciStart[0][4]
   #Event ending information
   lunar.end_date = ProdSciEnd[0][0]
   lunar.end_time = ProdSciEnd[0][1]
   lunar.end_latitude = ProdSciEnd[0][2]
   lunar.end_longitude = ProdSciEnd[0][3]
   lunar.end_altitude = ProdSciEnd[0][4]

   #Get information on the ground track during the event
   VDataGround = DataInterface.attach(Index)
   Index += 2
   for trk in range(lunar.num_grnd_trk):

      try:
         ProdGround = VDataGround.read()
      except HDF.HDF4Error:
         VDataGround.detach()
         print("Failed to read ground track information from the HDF file")
         return(-1,-1)

      lunar.gt_date[trk] = ProdGround[0][0]
      lunar.gt_time[trk] = ProdGround[0][1]
      lunar.gt_latitude[trk] = ProdGround[0][2]
      lunar.gt_longitude[trk] = ProdGround[0][3]
      lunar.gt_ray_dir[trk] = ProdGround[0][4]

   VDataGround.detach()


   #Get the temperature/pressure profiles used during processing
   VDataTrop = DataInterface.attach(Index)
   Index += 2
   try:
      ProdTrop = VDataTrop.read()
   except HDF.HDF4Error:
      VDataTrop.detach()
      print("Failed to read T/P profiles used during processessing from the HDF file")
      return(-1,-1)

   lunar.trop_temp = ProdTrop[0][0]
   lunar.trop_alt = ProdTrop[0][1]
   lunar.Met_Source = ProdTrop[0][2]

   VDataTrop.detach()


   #Get input meteoroligcal data
   VDataMetData = DataInterface.attach(Index)
   Index += 2

   for alt in range(lunar.num_press_grid):

      try:
         ProdMetData = VDataMetData.read()
      except HDF.HDF4Error:
         VDataMetData.detach()
         print("Failed to read science metadata from the HDF file")
         return(-1,-1)

      lunar.met_pressure[alt] = ProdMetData[0][0]
      lunar.met_temp[alt] = ProdMetData[0][1]
      lunar.met_temp_unc[alt] = ProdMetData[0][2]
      lunar.met_altitude[alt] = ProdMetData[0][3]

   VDataMetData.detach()


   #Get science meta data
   VDataSciData = DataInterface.attach(Index)
   Index += 2

   for alt in range(lunar.num_alt_bins):

      try:
         ProdSciData = VDataSciData.read()
      except HDF.HDF4Error:
         VDataSciData.detach()
         print("Failed to read science metadata from the HDF file")
         return(-1,-1)

      lunar.geom_alt[alt] = ProdSciData[0][0]
      lunar.AltRegQA[alt] = ProdSciData[0][1]
      lunar.potential_alt[alt] = ProdSciData[0][2]

   VDataSciData.detach()


   #Get the A-band altitude adjustment
   VDataAAdj = DataInterface.attach(Index)
   Index += 2
   try:
      ProdAAdj = VDataAAdj.read()
   except HDF.HDF4Error:
      VDataAAdj.detach()
      print("Failed to read T/P profiles used during processessing from the HDF file")
      return(-1,-1)

   lunar.AbandAltOffset = ProdAAdj[0][0]

   VDataAAdj.detach()


   #Get the temperature/pressure profiles used during processing
   VDataTempPres = DataInterface.attach(Index)
   Index += 2

   for alt in range(lunar.num_alt_bins):

      try:
         ProdTempPres = VDataTempPres.read()
      except HDF.HDF4Error:
         VDataTempPres.detach()
         print("Failed to read T/P profiles used during processessing from the HDF file")
         return(-1,-1)

      lunar.CurrentTemp[alt] = ProdTempPres[0][0]
      lunar.CurrentTemp_uncert[alt] = ProdTempPres[0][1]
      lunar.CurrentPress[alt] = ProdTempPres[0][2]
      lunar.CurrentPress_uncert[alt] = ProdTempPres[0][3]
      lunar.TPSource[alt] = ProdTempPres[0][4]

   VDataTempPres.detach()


   #Get the composite ozone product
   VDataO3Comp = DataInterface.attach(Index)
   Index += 2
   for alt in range(lunar.num_alt_bins):

      try:
         ProdO3Comp = VDataO3Comp.read()
      except HDF.HDF4Error:
         VDataO3Comp.detach()
         print("Failed to read the ozone profile from the HDF file")
         return(-1,-1)

      lunar.CompositeO3[alt] = ProdO3Comp[0][0]
      lunar.CompositeO3_uncert[alt] = ProdO3Comp[0][1]
      lunar.CompO3QA[alt] = ProdO3Comp[0][2]

   VDataO3Comp.detach()


   #Get the nitrogen dioxide product
   VDataNO2 = DataInterface.attach(Index)
   Index += 2
   for alt in range(lunar.num_alt_bins):

      try:
         ProdNO2 = VDataNO2.read()
      except HDF.HDF4Error:
         VDataNO2.detach()
         print("Failed to read the ozone profile from the HDF file")
         return(-1,-1)

      lunar.NO2NDP[alt] = ProdNO2[0][0]
      lunar.NO2NDP_uncert[alt] = ProdNO2[0][1]
      lunar.NO2QA[alt] = ProdNO2[0][2]

   VDataNO2.detach()


   #Get the nitrogen trioxide product
   VDataNO3 = DataInterface.attach(Index)
   Index += 2
   for alt in range(lunar.num_alt_bins):

      try:
         ProdNO3 = VDataNO3.read()
      except HDF.HDF4Error:
         VDataNO3.detach()
         print("Failed to read the Chappuis ozone product from the HDF file")
         return(-1,-1)

      lunar.NO3NDP[alt] = ProdNO3[0][0]
      lunar.NO3NDP_uncert[alt] = ProdNO3[0][1]
      lunar.NO3QA[alt] = ProdNO3[0][2]

   VDataNO3.detach()


   #Get the chlorine dioxide product
   VDataOClO = DataInterface.attach(Index)
   Index += 2
   for alt in range(lunar.num_alt_bins):

      try:
         ProdOClO = VDataOClO.read()
      except HDF.HDF4Error:
         VDataOClO.detach()
         print("Failed to read the aerosol ozone product from the HDF file")
         return(-1,-1)

      lunar.OClONDP[alt] = ProdOClO[0][0]
      lunar.OClONDP_uncert[alt] = ProdOClO[0][1]
      lunar.OClOQA[alt] = ProdOClO[0][2]

   VDataOClO.detach()


   DataInterface.end()
   FileHandle.close()

   return (lunar, 0)


########################################################################


def lun_l2_bin(Filename):
   """
   Name:        lun_l2_bin
   Author:      David Huber <david.b.huber@nasa.gov>
   Inputs:      Filename of a lunar level 2 binary product file
   Outputs:     1) A data object of type LunL2Data containing the product data
                2) A status flag indicating whether the read was successful or not
   Description: This function reads the data from a binary level 2 product file
                and returns a data object containing the data from the file.
                A status flag is also returned and should be checked before
                attempting to access the structure after function completion.  A
                status value of 0 indicates a success while a value of -1
                indicates that an error occurred.
   """

   #Import byteorder from sys to determine system endianess
   from sys import byteorder

   EndianFlag = -1
   order = byteorder

   if(order == "big"):
      EndianFlag = 0
      read_int32 = __b_read_int32
      read_float32 = __b_read_float32
   elif(order == "little"):
      EndianFlag = 1
      read_int32 = __l_read_int32
      read_float32 = __l_read_float32
   else:
      print("Unable to determine endianness of this machine\n")
      return (-1, -1)

   #Open the file
   File = open(Filename, "r")

   #Retrieve scalar information from the product file
   Event_ID = read_int32(File, 1)[0]
   yr_day_tag = read_int32(File, 1)[0]
   mission_time = read_float32(File, 1)[0]
   int_fill_value = read_int32(File, 1)[0]
   flt_fill_value = read_float32(File, 1)[0]
   mission_id = read_int32(File, 1)[0]
   L0DO_Version = read_float32(File, 1)[0]
   L0_Version = read_float32(File, 1)[0]
   Software_Version = read_float32(File, 1)[0]
   Dataproduct_Version = read_float32(File, 1)[0]
   DB_Version = read_float32(File, 1)[0]
   GRAM95_Version = read_float32(File, 1)[0]
   Met_Version = read_float32(File, 1)[0]
   Lun_Model_Ver = read_float32(File, 1)[0]
   Lun_Albedo_Ver = read_float32(File, 1)[0]
   bin_height = read_float32(File, 1)[0]
   num_alt_bins = read_int32(File, 1)[0]
   num_press_grid = read_int32(File, 1)[0]
   num_grnd_trk = read_int32(File, 1)[0]

   #Initialize the lunar object
   lunar = LunL2Data(num_grnd_trk, num_alt_bins, num_press_grid)

   #Fill the lunar object with the scalar data
   lunar.Event_ID = Event_ID
   lunar.yr_day_tag = yr_day_tag
   lunar.mission_time = mission_time
   lunar.int_fill_value = int_fill_value
   lunar.flt_fill_value = flt_fill_value
   lunar.mission_id = mission_id
   lunar.L0DO_Version = L0DO_Version
   lunar.L0_Version = L0_Version
   lunar.Software_Version = Software_Version
   lunar.Dataproduct_Version = Dataproduct_Version
   lunar.DB_Version = DB_Version
   lunar.GRAM95_Version = GRAM95_Version
   lunar.Met_Version = Met_Version
   lunar.Lun_Model_Ver = Lun_Model_Ver
   lunar.Lun_Albedo_Ver = Lun_Albedo_Ver
   lunar.bin_height = bin_height

   #Read the remainder of the data
   #Event type
   lunar.sc_evt_type = read_int32(File,1)[0]
   lunar.gnd_evt_type = read_int32(File,1)[0]
   #Beta angle
   lunar.BetaAngle = read_float32(File,1)[0]
   #Lunar phase
   lunar.LunarPhase = read_float32(File,1)[0]
   #Solar zenith angle
   lunar.ZenithAngle = read_float32(File,1)[0]
   lunar.aurora_flag = read_int32(File,1)[0]
   #Event status flag
   lunar.EvtStatusFlag = read_int32(File,1)[0]
   #Starting time/position
   lunar.st_date = read_int32(File,1)[0]
   lunar.st_time = read_int32(File,1)[0]
   lunar.st_latitude = read_float32(File,1)[0]
   lunar.st_longitude = read_float32(File,1)[0]
   lunar.st_altitude = read_float32(File,1)[0]
   #Ending time/position
   lunar.end_date = read_int32(File,1)[0]
   lunar.end_time = read_int32(File,1)[0]
   lunar.end_latitude = read_float32(File,1)[0]
   lunar.end_longitude = read_float32(File,1)[0]
   lunar.end_altitude = read_float32(File,1)[0]
   #Ground track information
   lunar.gt_date = read_int32(File,lunar.num_grnd_trk)
   lunar.gt_time = read_int32(File,lunar.num_grnd_trk)
   lunar.gt_latitude = read_float32(File,lunar.num_grnd_trk)
   lunar.gt_longitude = read_float32(File,lunar.num_grnd_trk)
   lunar.gt_ray_dir = read_float32(File,lunar.num_grnd_trk)
   #Tropopause temperature/altitude
   lunar.trop_temp = read_float32(File,1)[0]
   lunar.trop_alt = read_float32(File,1)[0]
   #Meteorological inputs
   lunar.met_pressure = read_float32(File,lunar.num_press_grid)
   lunar.met_temp = read_float32(File,lunar.num_press_grid)
   lunar.met_temp_unc = read_float32(File,lunar.num_press_grid)
   lunar.met_altitude = read_float32(File,lunar.num_press_grid)
   lunar.Met_Source = read_int32(File,1)[0]
   #Altitude registration
   lunar.geom_alt = read_float32(File,lunar.num_alt_bins)
   lunar.AltRegQA = read_int32(File,lunar.num_alt_bins)
   lunar.potential_alt = read_float32(File,lunar.num_alt_bins)
   #A-band altitude adjustment
   lunar.AbandAltOffset = read_float32(File,1)[0]
   #Temperature/pressure profiles
   lunar.CurrentTemp = read_float32(File,lunar.num_alt_bins)
   lunar.CurrentTemp_uncert = read_float32(File,lunar.num_alt_bins)
   lunar.CurrentPress = read_float32(File,lunar.num_alt_bins)
   lunar.CurrentPress_uncert = read_float32(File,lunar.num_alt_bins)
   lunar.TPSource = read_int32(File,lunar.num_alt_bins)
   #Composite O3 products
   lunar.CompositeO3 = read_float32(File,lunar.num_alt_bins)
   lunar.CompositeO3_uncert = read_float32(File,lunar.num_alt_bins)
   lunar.CompO3QA = read_int32(File,lunar.num_alt_bins)
   #Nitrogen Dioxide products
   lunar.NO2NDP = read_float32(File,lunar.num_alt_bins)
   lunar.NO2NDP_uncert = read_float32(File,lunar.num_alt_bins)
   lunar.NO2QA = read_int32(File,lunar.num_alt_bins)
   #Nitrogen Trioxide products
   lunar.NO3NDP = read_float32(File,lunar.num_alt_bins)
   lunar.NO3NDP_uncert = read_float32(File,num_alt_bins)
   lunar.NO3QA = read_int32(File,lunar.num_alt_bins)
   #Chlorine Dioxide products
   lunar.OClONDP = read_float32(File,lunar.num_alt_bins)
   lunar.OClONDP_uncert = read_float32(File,lunar.num_alt_bins)
   lunar.OClOQA = read_int32(File,lunar.num_alt_bins)

   File.close()
   return(lunar,0)
