pro read_lunar_hdf, filename, status, lunar

;  Purpose:
;  This program provides the data structures necessary to read the SAGE III
;  Lunar Event Species data hdf product file.
;
;  Author:
;  Larry Selvy 
;  
;  Contact:
;  michael.l.heitz@nasa.gov 
;
;  Initial Version:
;  11/09/11
;
;  Input Variables:
;  filename - The name of the lunar species hdf product file (g3a.lsp.*)
;
;  Output Variables;
;  status - status of the read file routine
;  lunar - Structure containing all the data from the lunar HDF product file

status = 'OK'

;Create the structure to read the data for dynamic array allocation

lunar_size = { lunar_size, event_id:0L, $
          yr_day_tag:0L, $
          mission_time:0., $
          int_fill_value:0L, $
          flt_fill_value:0., $
          mission_id:0L, $
          L0DO_Version:0., $
          L0_Version:0., $
          Software_Version:0., $
          Dataproduct_Version:0., $
          DB_Version:0., $
          GRAM95_Version:0., $
          Met_Version:0., $
          Lun_Model_Ver:0., $
          Lun_Albedo_Ver:0., $
          bin_height:0., $
          num_alt_bins:0L, $
          num_press_grid:0L, $
          num_grnd_trk:0L }

!QUIET = 1

;  Open the file
fileID = HDF_OPEN(filename,/read)

;  Get to the first vdata
vdID = -1
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parse the byte array returned into intellegible data
lunar_size.event_id = long(x,0)
lunar_size.yr_day_tag = long(x,4)
lunar_size.mission_time = float(x,8)
lunar_size.int_fill_value = long(x,12)
lunar_size.flt_fill_value = float(x,16)
lunar_size.mission_id = long(x,20)
;  Detach this vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information from this byte array into understandable data
lunar_size.L0DO_Version = float(x,0)
lunar_size.L0_Version = float(x,4)
lunar_size.DB_Version = float(x,8)
lunar_size.GRAM95_Version = float(x,12)
lunar_size.Met_Version = float(x,16)
lunar_size.Software_Version = float(x,20)
lunar_size.Dataproduct_Version = float(x,24)
lunar_size.Lun_Model_Ver = float(x,28)
lunar_size.Lun_Albedo_Ver = float(x,32)
;  Detach this Vdata
HDF_VD_DETACH,vdHandle

;  Get the next vdata
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle, x)
if nrecs ne 1 then begin
   print,'There is an error in the HDF file ',vdID
   return
endif
;  Parse the information fromt he Byte array to understandable data
lunar_size.bin_height = float(x,0)
lunar_size.num_alt_bins = long(x,4)
lunar_size.num_grnd_trk = long(x,8)
lunar_size.num_press_grid = long(x,12)
;  Detach the vdata
HDF_VD_DETACH,vdHandle

;  Set version variable
vers = lunar_size.Software_Version

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;  This is the data structure for the data read in from the HDF file ;
;  IT IS TO BE NOTED THAT THIS IS VERSION 3 ONLY                     ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
lunar = { Event_ID:0L, $
     yr_day_tag:0L, $
     mission_time:0., $
     int_fill_value:0L, $
     flt_fill_value:0., $
     mission_id:0L, $
     L0DO_Version:0., $
     L0_Version:0., $
     Software_Version:0., $
     Dataproduct_Version:0., $
     DB_Version:0., $
     GRAM95_Version:0., $
     Met_Version:0., $
     Lun_Model_Ver:0., $
     Lun_Albedo_Ver:0., $
     bin_height:0., $
     num_alt_bins:0L, $
     num_press_grid:0L, $
     num_grnd_trk:0L, $
     sc_evt_type:0L, $
     gnd_evt_type:0L, $
     BetaAngle:0., $
     LunarPhase:0., $
     ZenithAngle:0., $
     aurora_flag:0L, $
     EvtStatusFlag:0L, $
     st_date:0L, $
     st_time:0L, $
     st_latitude:0., $
     st_longitude:0., $
     st_altitude:0., $
     end_date:0L, $
     end_time:0L, $
     end_latitude:0., $
     end_longitude:0., $
     end_altitude:0., $
     gt_date:lonarr(lunar_size.num_grnd_trk), $
     gt_time:lonarr(lunar_size.num_grnd_trk), $
     gt_latitude:fltarr(lunar_size.num_grnd_trk), $
     gt_longitude:fltarr(lunar_size.num_grnd_trk), $
     gt_ray_dir:fltarr(lunar_size.num_grnd_trk), $
     trop_temp:0., $
     trop_alt:0., $
     met_pressure:fltarr(lunar_size.num_press_grid), $
     met_temp:fltarr(lunar_size.num_press_grid), $
     met_temp_unc:fltarr(lunar_size.num_press_grid), $
     met_altitude:fltarr(lunar_size.num_press_grid), $
     Met_Source:0L, $
     geom_alt:fltarr(lunar_size.num_alt_bins), $
     AltRegQA:lonarr(lunar_size.num_alt_bins), $
     potential_alt:fltarr(lunar_size.num_alt_bins), $
     AbandAltRegOffset:0., $
     CurrentTemp:fltarr(lunar_size.num_alt_bins), $
     CurrentTemp_uncert:fltarr(lunar_size.num_alt_bins), $
     CurrentPress:fltarr(lunar_size.num_alt_bins), $
     CurrentPress_uncert:fltarr(lunar_size.num_alt_bins), $
     TPSource:lonarr(lunar_size.num_alt_bins), $
     CompositeO3:fltarr(lunar_size.num_alt_bins), $
     CompositeO3_uncert:fltarr(lunar_size.num_alt_bins), $
     CompO3QA:lonarr(lunar_size.num_alt_bins), $
     NO2NDP:fltarr(lunar_size.num_alt_bins), $
     NO2NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO2QA:lonarr(lunar_size.num_alt_bins), $
     NO3NDP:fltarr(lunar_size.num_alt_bins), $
     NO3NDP_uncert:fltarr(lunar_size.num_alt_bins), $
     NO3QA:lonarr(lunar_size.num_alt_bins), $
     OClONDP:fltarr(lunar_size.num_alt_bins), $
     OClONDP_uncert:fltarr(lunar_size.num_alt_bins), $
     OClOQA:lonarr(lunar_size.num_alt_bins) }

;  Now we read in the data from the lunar_size array
lunar.event_id = lunar_size.event_id
lunar.yr_day_tag = lunar_size.yr_day_tag
lunar.mission_time = lunar_size.mission_time
lunar.int_fill_value = lunar_size.int_fill_value
lunar.flt_fill_value = lunar_size.flt_fill_value
lunar.mission_id = lunar_size.mission_id
lunar.L0DO_Version = lunar_size.L0DO_Version
lunar.L0_Version = lunar_size.L0_Version
lunar.Software_Version = lunar_size.Software_Version
lunar.Dataproduct_Version = lunar_size.Dataproduct_Version
lunar.DB_Version = lunar_size.DB_Version
lunar.GRAM95_Version = lunar_size.GRAM95_Version
lunar.Met_Version = lunar_size.Met_Version
lunar.bin_height = lunar_size.bin_height
lunar.num_alt_bins = lunar_size.num_alt_bins
lunar.num_grnd_trk = lunar_size.num_grnd_trk
lunar.num_press_grid = lunar_size.num_press_grid
lunar.Lun_Model_Ver = lunar_size.Lun_Model_Ver
lunar.Lun_Albedo_Ver = lunar_size.Lun_Albedo_Ver

;Now we continue to read in the rest of the data from the HDF file
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
lunar.sc_evt_type = long(x,0)
lunar.gnd_evt_type = long(x,4)
lunar.BetaAngle = float(x,8)
lunar.LunarPhase = float(x,12)
lunar.ZenithAngle = float(x,16)
lunar.aurora_flag = long(x,20)
lunar.EvtStatusFlag = long(x,24)
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
lunar.st_date = long(x,0)
lunar.st_time = long(x,4)
lunar.st_latitude = float(x,8)
lunar.st_longitude = float(x,12)
lunar.st_altitude = float(x,16)
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
lunar.end_date = long(x,0)
lunar.end_time = long(x,4)
lunar.end_latitude = float(x,8)
lunar.end_longitude = float(x,12)
lunar.end_altitude = float(x,16)
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_grnd_trk then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k=0L, lunar_size.num_grnd_trk - 1 DO BEGIN
   lunar.gt_date(k) = long(x,index)
   index = index + 4
   lunar.gt_time(k) = long(x,index)
   index = index + 4
   lunar.gt_latitude(k) = float(x,index)
   index = index + 4
   lunar.gt_longitude(k) = float(x,index)
   index = index + 4
   lunar.gt_ray_dir(k) = float(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
lunar.trop_temp = float(x,0)
lunar.trop_alt = float(x,4)
lunar.Met_Source = long(x,8)
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_press_grid then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_press_grid - 1 DO BEGIN
   lunar.met_pressure(k) = float(x,index)
   index = index + 4
   lunar.met_temp(k) = float(x,index)
   index = index + 4
   lunar.met_temp_unc(k) = float(x,index)
   index = index + 4
   lunar.met_altitude(k) = float(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.geom_alt(k) = float(x,index)
   index = index + 4
   lunar.AltRegQA(k) = long(x,index)
   index = index + 4
   lunar.potential_alt(k) = float(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne 1 then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
lunar.AbandAltRegOffset = float(x,0)
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.CurrentTemp(k) = float(x,index)
   index = index + 4
   lunar.CurrentTemp_uncert(k) = float(x,index)
   index = index + 4
   lunar.CurrentPress(k) = float(x,index)
   index = index + 4
   lunar.CurrentPress_uncert(k) = float(x,index)
   index = index + 4
   lunar.TPSource(k) = long(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.CompositeO3(k) = float(x,index)
   index = index + 4
   lunar.CompositeO3_uncert(k) = float(x,index)
   index = index + 4
   lunar.CompO3QA(k) = long(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.NO2NDP(k) = float(x,index)
   index = index + 4
   lunar.NO2NDP_uncert(k) = float(x,index)
   index = index + 4
   lunar.NO2QA(k) = long(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.NO3NDP(k) = float(x,index)
   index = index + 4
   lunar.NO3NDP_uncert(k) = float(x,index)
   index = index + 4
   lunar.NO3QA(k) = long(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle
vdID = HDF_VD_GETID(fileID,vdID)
vdHandle = HDF_VD_ATTACH(fileID,vdID,/read)
nrecs = HDF_VD_READ(vdHandle,x)
if nrecs ne lunar_size.num_alt_bins then begin
   print,"There is an error in the HDF file ",vdID
   return
endif
;Parsing the data from the retrieved byte array into intelligble data
index = 0
for k = 0L, lunar_size.num_alt_bins - 1 DO BEGIN
   lunar.OClONDP(k) = float(x,index)
   index = index + 4
   lunar.OClONDP_uncert(k) = float(x,index)
   index = index + 4
   lunar.OClOQA(k) = long(x,index)
   index = index + 4
endfor
;Close the Vdata
HDF_VD_DETACH,vdHandle

;  This is the end of the data and the HDF file, close the file
HDF_CLOSE,fileID

end

